@@ -203,15 +203,12 @@ gossip_chatroom_manager_add (GossipChatroomManager *manager,
 
 	/* don't add more than once */
 	if (!gossip_chatroom_manager_find (manager, gossip_chatroom_get_id (chatroom))) {
-		const gchar        *name;
-		GossipChatroomType  type;
+		const gchar *name;
 
-		type = gossip_chatroom_get_type (chatroom);
 		name = gossip_chatroom_get_name (chatroom);
 
-		gossip_debug (DEBUG_DOMAIN, "Adding %s%s chatroom with name:'%s'",
+		gossip_debug (DEBUG_DOMAIN, "Adding %s chatroom with name:'%s'",
 			      gossip_chatroom_get_auto_connect (chatroom) ? "connecting on startup " : "",
-			      gossip_chatroom_type_to_string (type),
 			      name);
 
 		g_signal_connect (chatroom, "notify::enabled",
@@ -265,6 +262,26 @@ gossip_chatroom_manager_remove (GossipChatroomManager *manager,
 	}
 }
 
+void
+gossip_chatroom_manager_set_index (GossipChatroomManager *manager,
+				   GossipChatroom        *chatroom,
+				   gint                   index)
+{
+	GossipChatroomManagerPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHATROOM_MANAGER (manager));
+	g_return_if_fail (GOSSIP_IS_CHATROOM (chatroom));
+
+	priv = GET_PRIV (manager);
+
+	gossip_debug (DEBUG_DOMAIN,
+		      "Indexing chatroom with name:'%s' to position %d",
+		      gossip_chatroom_get_name (chatroom), index);
+
+	priv->chatrooms = g_list_remove (priv->chatrooms, chatroom);
+	priv->chatrooms = g_list_insert (priv->chatrooms, chatroom, index);
+}
+
 static void
 chatroom_manager_chatroom_enabled_cb (GossipChatroom        *chatroom,
 				      GParamSpec            *arg1,
@@ -537,15 +554,13 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 	GossipChatroom            *chatroom;
 	xmlNodePtr                 child;
 	gchar                     *str;
-	GossipChatroomType         type;
 	gchar                     *name, *nick, *server;
 	gchar                     *room, *password, *account_name;
 	gboolean                   auto_connect, favourite;
 
 	priv = GET_PRIV (manager);
 
-	/* default values. */
-	type = GOSSIP_CHATROOM_TYPE_NORMAL;
+	/* Default values. */
 	name = NULL;
 	nick = NULL;
 	server = NULL;
@@ -611,7 +626,6 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 
 	if (name && server && room) {
 		chatroom = g_object_new (GOSSIP_TYPE_CHATROOM,
-					 "type", type,
 					 "name", name,
 					 "server", server,
 					 "room", room,
@@ -761,20 +775,12 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 	for (l = chatrooms; l; l = l->next) {
 		GossipChatroom *chatroom;
 		GossipAccount  *account;
-		gchar          *type;
 		xmlNodePtr      node;
 
 		chatroom = l->data;
 
-		switch (gossip_chatroom_get_type (chatroom)) {
-		case GOSSIP_CHATROOM_TYPE_NORMAL:
-		default:
-			type = g_strdup_printf ("normal");
-			break;
-		}
-
 		node = xmlNewChild (root, NULL, "chatroom", NULL);
-		xmlNewChild (node, NULL, "type", type);
+		xmlNewChild (node, NULL, "type", "normal"); /* We should remove this */
 		xmlNewTextChild (node, NULL, "name", gossip_chatroom_get_name (chatroom));
 		xmlNewTextChild (node, NULL, "nick", gossip_chatroom_get_nick (chatroom));
 
@@ -783,14 +789,12 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 
 		xmlNewTextChild (node, NULL, "password", gossip_chatroom_get_password (chatroom));
 		xmlNewChild (node, NULL, "auto_connect", gossip_chatroom_get_auto_connect (chatroom) ? "yes" : "no");
-		xmlNewChild (node, NULL, "favourite", gossip_chatroom_get_is_favourite (chatroom) ? "yes" : "no");
+		xmlNewChild (node, NULL, "favourite", gossip_chatroom_get_favourite (chatroom) ? "yes" : "no");
 
 		account = gossip_chatroom_get_account (chatroom);
 		if (account) {
 			xmlNewTextChild (node, NULL, "account", gossip_account_get_name (account));
 		}
-
-		g_free (type);
 	}
 
 	/* Make sure the XML is indented properly */
