@@ -30,6 +30,10 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include <ige-mac-menu.h>
+#endif
+
 #include <libgossip/gossip-chatroom.h>
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-event.h>
@@ -124,6 +128,8 @@ struct _GossipAppPriv {
 	/* Tooltips for all widgets */
 	GtkTooltips           *tooltips;
 
+	GtkWidget             *menu_bar;
+
 	/* Menu widgets */
 	GtkWidget             *chat_connect;
 	GtkWidget             *chat_disconnect;
@@ -178,6 +184,10 @@ static gboolean
 app_main_window_key_press_event_cb           (GtkWidget             *window,
 					      GdkEventKey           *event,
 					      GossipApp             *app);
+static gboolean
+app_main_window_focus_in_event_cb            (GtkWidget             *widget,
+					      GdkEvent              *event,
+					      GossipApp             *app);
 static void     app_find_entry_changed_cb    (GtkEditable           *editable,
 					      GossipApp             *app);
 static void     
@@ -207,10 +217,6 @@ static void
 app_favorite_chatroom_menu_setup             (void);
 static void     app_chat_quit_cb             (GtkWidget             *window,
 					      GossipApp             *app);
-static void     app_chat_connect_cb          (GtkWidget             *window,
-					      GossipApp             *app);
-static void     app_chat_disconnect_cb       (GtkWidget             *window,
-					      GossipApp             *app);
 static void     app_chat_search_cb           (GtkWidget             *window,
 					      GossipApp             *app);
 static void     app_chat_new_message_cb      (GtkWidget             *widget,
@@ -299,7 +305,6 @@ static void
 app_notify_compact_contact_list_cb           (GossipConf            *conf,
 					      const gchar           *key,
 					      gpointer               user_data);
-static void     app_disconnect               (void);
 static void     app_connection_items_setup   (GladeXML              *glade);
 static void     app_connection_items_update  (void);
 static void     
@@ -331,6 +336,9 @@ app_session_chatroom_auto_connect_cb         (GossipSession         *session,
 static void     app_event_added_cb           (GossipEventManager    *manager,
 					      GossipEvent           *event,
 					      gpointer               user_data);
+static void     app_accounts_updated_cb      (GossipAccountManager  *manager,
+					      GossipAccount         *account,
+					      gpointer               user_data);
 static void     app_contact_activated_cb     (GossipContactList     *contact_list,
 					      GossipContact         *contact,
 					      GossipEventId          event_id,
@@ -445,6 +453,10 @@ app_finalize (GObject *object)
 					      app_event_added_cb,
 					      NULL);
 
+	g_signal_handlers_disconnect_by_func (gossip_session_get_account_manager (priv->session),
+					      app_accounts_updated_cb,
+					      NULL);
+
 	gossip_ft_dialog_finalize (priv->session);
 	gossip_subscription_dialog_finalize (priv->session);
 	
@@ -693,13 +705,14 @@ app_setup_contact_list (GtkWidget *scrolled_window,
 static void
 app_setup (GossipSession *session)
 {
-	GossipAppPriv *priv;
-	GossipConf    *conf;
-	GladeXML      *glade;
-	GtkWidget     *sw;
-	GtkWidget     *show_offline_widget;
-	GtkWidget     *help_contents;
-
+	GossipAppPriv        *priv;
+	GossipConf           *conf;
+	GossipAccountManager *account_manager;
+	GladeXML             *glade;
+	GtkWidget            *sw;
+	GtkWidget            *show_offline_widget;
+	GtkWidget            *help_contents;
+			
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Beginning...");
 
 	priv = GET_PRIV (app);
@@ -708,6 +721,8 @@ app_setup (GossipSession *session)
 
 	priv->session = g_object_ref (session);
 
+	account_manager = gossip_session_get_account_manager (session);
+
 	gossip_debug (DEBUG_DOMAIN_SETUP,
 		      "Initialising session listeners "
 		      "(subscription, file transfer, etc)");
@@ -744,19 +759,15 @@ app_setup (GossipSession *session)
 	g_signal_connect (priv->session, "protocol-connecting",
 			  G_CALLBACK (app_session_protocol_connecting_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "protocol-connected",
 			  G_CALLBACK (app_session_protocol_connected_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "protocol-disconnected",
 			  G_CALLBACK (app_session_protocol_disconnected_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "protocol-error",
 			  G_CALLBACK (app_session_protocol_error_cb),
 			  NULL);
-
 	g_signal_connect (priv->session, "chatroom-auto-connect",
 			  G_CALLBACK (app_session_chatroom_auto_connect_cb),
 			  NULL);
@@ -765,6 +776,13 @@ app_setup (GossipSession *session)
 			  G_CALLBACK (app_event_added_cb),
 			  NULL);
 
+	g_signal_connect (account_manager, "account-added",
+			  G_CALLBACK (app_accounts_updated_cb),
+			  NULL);
+	g_signal_connect (account_manager, "account-removed",
+			  G_CALLBACK (app_accounts_updated_cb),
+			  NULL);
+
 	/* Set up interface */
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Initialising interface");
 	glade = gossip_glade_get_file ("main.glade",
@@ -772,6 +790,7 @@ app_setup (GossipSession *session)
 				       NULL,
 				       "main_window", &priv->window,
 				       "main_vbox", &priv->main_vbox,
+				       "main_menubar", &priv->menu_bar,
 				       "find_hbox", &priv->find_hbox,
 				       "find_entry", &priv->find_entry,
 				       "errors_vbox", &priv->errors_vbox,
@@ -795,11 +814,10 @@ app_setup (GossipSession *session)
 			      "main_window", "delete_event", app_main_window_delete_event_cb,
 			      "main_window", "configure_event", app_window_configure_event_cb,
 			      "main_window", "key_press_event", app_main_window_key_press_event_cb,
+			      "main_window", "focus-in-event", app_main_window_focus_in_event_cb,
 			      "find_entry", "changed", app_find_entry_changed_cb, 
 			      "chat", "button-press-event", app_chat_button_press_event_cb,
 			      "chat_quit", "activate", app_chat_quit_cb,
-			      "chat_connect", "activate", app_chat_connect_cb,
-			      "chat_disconnect", "activate", app_chat_disconnect_cb,
 			      "chat_search", "activate", app_chat_search_cb,
 			      "chat_new_message", "activate", app_chat_new_message_cb,
 			      "chat_history", "activate", app_chat_history_cb,
@@ -818,6 +836,39 @@ app_setup (GossipSession *session)
 	/* Set up menu */
 	app_favorite_chatroom_menu_setup ();
 
+#ifdef GDK_WINDOWING_QUARTZ
+	{
+		GtkWidget       *item;
+		IgeMacMenuGroup *group;
+
+		gtk_widget_set_name (priv->window, "small-font");
+
+		ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (priv->menu_bar));
+
+		item = glade_xml_get_widget (glade, "chat_quit");
+		ige_mac_menu_set_quit_menu_item (GTK_MENU_ITEM (item));
+
+		group =  ige_mac_menu_add_app_menu_group ();
+		item = glade_xml_get_widget (glade, "help_about");
+		ige_mac_menu_add_app_menu_item (group, GTK_MENU_ITEM (item),
+						_("About Gossip"));
+
+		group =  ige_mac_menu_add_app_menu_group ();
+		item = glade_xml_get_widget (glade, "edit_preferences");
+		ige_mac_menu_add_app_menu_item (group, GTK_MENU_ITEM (item),
+						_("Preferences..."));
+
+		/* Hide the left over separators. */
+		item = glade_xml_get_widget (glade, "quit_separator");
+		gtk_widget_hide (item);
+
+		item = glade_xml_get_widget (glade, "preferences_separator");
+		gtk_widget_hide (item);
+
+		gtk_widget_hide (priv->menu_bar);
+	}
+#endif
+
 	gtk_widget_hide (priv->chat_context);
 	gtk_widget_hide (priv->chat_context_separator);
 
@@ -836,7 +887,6 @@ app_setup (GossipSession *session)
 	
 	app_setup_throbber ();
 
-
 	/* Set up notification area / tray. */
 	gossip_debug (DEBUG_DOMAIN_SETUP, "Configuring notification area widgets");
 	app_status_icon_create_menu ();
@@ -1070,6 +1120,22 @@ app_main_window_key_press_event_cb (GtkWidget   *window,
 	return FALSE;
 }
 
+static gboolean
+app_main_window_focus_in_event_cb (GtkWidget *window,
+				   GdkEvent  *event,
+				   GossipApp *app)
+{
+	GossipAppPriv *priv;
+
+	priv = GET_PRIV (app);
+
+#ifdef GDK_WINDOWING_QUARTZ
+	ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (priv->menu_bar));
+#endif
+
+	return FALSE;
+}
+
 static void
 app_find_entry_changed_cb (GtkEditable *editable,
 		     	   GossipApp   *app)
@@ -1260,6 +1326,13 @@ app_favorite_chatroom_menu_setup (void)
 	priv->room_menu =
 		gtk_menu_item_get_submenu (GTK_MENU_ITEM (priv->room));
 
+	/* Reverse the list here because we want to make sure that we
+	 * always insert the next item in at position 3 so we start at
+	 * the right place. This of course means that new items are
+	 * put in before the last one.
+	 */
+	chatrooms = g_list_reverse (chatrooms);
+
 	for (l = chatrooms; l; l = l->next) {
 		if (app_favorite_chatroom_menu_add (l->data)) {
 			found = TRUE;
@@ -1285,6 +1358,26 @@ app_favorite_chatroom_menu_setup (void)
 			  NULL);
 }
 
+static void
+app_chat_connect_activate_cb (GtkMenuItem *menuitem,
+			      gpointer     user_data)
+{
+	GossipAccount *account;
+
+	account = g_object_get_data (G_OBJECT (menuitem), "account");
+	gossip_app_connect (account, FALSE); 
+}
+
+static void
+app_chat_disconnect_activate_cb (GtkMenuItem *menuitem,
+				 gpointer     user_data)
+{
+	GossipAccount *account;
+
+	account = g_object_get_data (G_OBJECT (menuitem), "account");
+	gossip_app_disconnect (account); 
+}
+
 static void
 app_chat_quit_cb (GtkWidget *window,
 		  GossipApp *app)
@@ -1298,20 +1391,6 @@ app_chat_quit_cb (GtkWidget *window,
 	}
 }
 
-static void
-app_chat_connect_cb (GtkWidget *window,
-		     GossipApp *app)
-{
-	gossip_app_connect (NULL, FALSE);
-}
-
-static void
-app_chat_disconnect_cb (GtkWidget *window,
-			GossipApp *app)
-{
-	app_disconnect ();
-}
-
 static void
 app_chat_search_cb (GtkWidget *window,
 		    GossipApp *app)
@@ -1596,10 +1675,10 @@ app_session_protocol_connected_cb (GossipSession *session,
 static void
 app_reconnect_remove (gpointer data)
 {
-	guint *id;
+	guint id;
 
-	id = (guint*) data;
-	g_source_remove (*id);
+	id = GPOINTER_TO_UINT (data);
+	g_source_remove (id);
 }
 
 static gboolean
@@ -1670,7 +1749,7 @@ app_session_protocol_disconnected_cb (GossipSession *session,
 				    account);
 		g_hash_table_insert (priv->reconnects,
 				     g_object_ref (account),
-				     &id);
+				     GUINT_TO_POINTER (id));
 	}
 }
 
@@ -2204,6 +2283,23 @@ gossip_app_connect (GossipAccount *account, gboolean startup)
 	gossip_session_connect (priv->session, account, startup);
 }
 
+void
+gossip_app_disconnect (GossipAccount *account)
+{
+	GossipAppPriv        *priv;
+	GossipAccountManager *manager;
+	
+	priv = GET_PRIV (app);
+
+	manager = gossip_session_get_account_manager (priv->session);
+	if (gossip_account_manager_get_count (manager) < 1) {
+		/* Show the accounts dialog instead */
+		return;
+	}
+	
+	gossip_session_disconnect (priv->session, account);
+}
+
 /* Test hack to add support for disconnecting all accounts and then connect
  * them again due to if the net goes up and down.
  */
@@ -2283,16 +2379,6 @@ gossip_app_get (void)
 	return app;
 }
 
-static void
-app_disconnect (void)
-{
-	GossipAppPriv *priv;
-
-	priv = GET_PRIV (app);
-
-	gossip_session_disconnect (priv->session, NULL);
-}
-
 static void
 app_connection_items_setup (GladeXML *glade)
 {
@@ -2300,7 +2386,8 @@ app_connection_items_setup (GladeXML *glade)
 
 	const gchar   *widgets_connected[] = {
 		"chat_disconnect",
-		"room",
+		"room_join_new", 
+		"room_join_favorites", 
 		"chat_new_message",
 		"chat_add_contact",
 		"edit_personal_information"
@@ -2331,6 +2418,148 @@ app_connection_items_setup (GladeXML *glade)
 	priv->widgets_disconnected = list;
 }
 
+static void
+app_connection_menu_update (void)
+{
+	GossipAppPriv        *priv;
+	GossipAccountManager *manager;
+	GList                *accounts;
+	GList                *l;
+	GtkWidget            *menu_connect = NULL;
+	GtkWidget            *menu_disconnect = NULL;
+	gboolean              connected;
+
+	priv = GET_PRIV (app);
+		
+	manager = gossip_session_get_account_manager (priv->session);
+	accounts = gossip_account_manager_get_accounts (manager);
+
+	/* We always disconnect these in case we move from 1 account
+	 * to many accounts, if we don't do this, we start connecting
+	 * when we are showing submenus.
+	 */
+	g_signal_handlers_disconnect_by_func (priv->chat_connect, 
+					      app_chat_connect_activate_cb,
+					      NULL);
+	g_signal_handlers_disconnect_by_func (priv->chat_disconnect, 
+					      app_chat_disconnect_activate_cb,
+					      NULL);
+
+	/* If we have one account we do things quite differently. We
+	 * only have one menu item and toggle it.
+	 */
+	if (g_list_length (accounts) == 0) {
+		gtk_widget_show (priv->chat_connect);
+		gtk_widget_hide (priv->chat_disconnect);
+
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_connect), NULL);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_disconnect), NULL);
+
+		g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+		g_list_free (accounts);
+
+		return;
+	}
+
+	if (g_list_length (accounts) == 1) {
+		connected = gossip_session_is_connected (priv->session, accounts->data);
+
+		if (connected) {
+			gtk_widget_hide (priv->chat_connect);
+			gtk_widget_show (priv->chat_disconnect);
+		} else {
+			gtk_widget_show (priv->chat_connect);
+			gtk_widget_hide (priv->chat_disconnect);
+		}			
+
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_connect), NULL);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_disconnect), NULL);
+
+		g_object_set_data_full (G_OBJECT (priv->chat_connect), "account", 
+					g_object_ref (accounts->data), 
+					g_object_unref);
+		g_object_set_data_full (G_OBJECT (priv->chat_disconnect), "account", 
+					g_object_ref (accounts->data), 
+					g_object_unref);
+
+		g_signal_connect (priv->chat_connect, "activate",
+				  G_CALLBACK (app_chat_connect_activate_cb),
+				  NULL);
+		g_signal_connect (priv->chat_disconnect, "activate",
+				  G_CALLBACK (app_chat_disconnect_activate_cb),
+				  NULL);
+
+		g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+		g_list_free (accounts);
+
+		return;
+	}
+
+	/* If we have multiple accounts we show a connect-> and a
+	 * disconnect-> menu instead. 
+	 */
+	gtk_widget_show (priv->chat_connect);
+	gtk_widget_show (priv->chat_disconnect);
+
+	for (l = accounts; l; l = l->next) {
+		GtkWidget   *item;
+		GtkWidget   *image;
+		GCallback    callback;
+		const gchar *name;
+		GdkPixbuf   *pixbuf;
+
+		name = gossip_account_get_name (l->data);
+		item = gtk_image_menu_item_new_with_label (name);
+
+		connected = gossip_session_is_connected (priv->session, l->data);
+		pixbuf = gossip_account_status_create_pixbuf (l->data, 
+							      GTK_ICON_SIZE_MENU,
+							      connected);
+		image = gtk_image_new_from_pixbuf (pixbuf);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_object_unref (pixbuf);
+
+		gtk_widget_show (item);
+
+		if (gossip_session_is_connected (priv->session, l->data) || 
+		    gossip_session_is_connecting (priv->session, l->data)) {
+			/* Create disconnect account menu item */
+			if (!menu_disconnect) {
+				menu_disconnect = gtk_menu_new ();
+				gtk_widget_show (menu_disconnect);
+			}
+
+			callback = G_CALLBACK (app_chat_disconnect_activate_cb);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu_disconnect), item);
+		} else {
+			/* Create connect account menu item */
+			if (!menu_connect) {
+				menu_connect = gtk_menu_new ();
+				gtk_widget_show (menu_connect);
+			}
+			
+			callback = G_CALLBACK (app_chat_connect_activate_cb);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu_connect), item);
+		}
+
+		g_object_set_data_full (G_OBJECT (item), "account", 
+					g_object_ref (l->data), 
+					g_object_unref);
+		g_signal_connect (item, "activate",
+				  callback,
+				  NULL);
+	}
+
+	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+	g_list_free (accounts);
+
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_connect), menu_connect);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->chat_disconnect), menu_disconnect);
+
+	gtk_widget_set_sensitive (priv->chat_connect, menu_connect != NULL);
+	gtk_widget_set_sensitive (priv->chat_disconnect, menu_disconnect != NULL);
+}
+
 static void
 app_connection_items_update (void)
 {
@@ -2341,6 +2570,9 @@ app_connection_items_update (void)
 
 	priv = GET_PRIV (app);
 
+	/* Set up connect/disconnect menus */
+	app_connection_menu_update ();
+
 	/* Get account count for:
 	 *  - connected and disabled,
 	 *  - connected and enabled
@@ -2671,6 +2903,14 @@ app_event_added_cb (GossipEventManager *manager,
 	gossip_request_user_attention ();
 }
 
+static void
+app_accounts_updated_cb (GossipAccountManager  *manager,
+			 GossipAccount         *account,
+			 gpointer               user_data)
+{
+	app_connection_items_update ();
+}
+
 static void
 app_contact_activated_cb (GossipContactList *contact_list,
 			  GossipContact     *contact,
