@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,17 +16,24 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+ 
 #include <stdlib.h>
 #include <string.h>
+
 #include <dbus/dbus-glib-bindings.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-utils.h>
 
+#include "gossip-chat-manager.h"
 #include "gossip-app.h"
 #include "gossip-dbus.h"
 #include "gossip-new-message-dialog.h"
@@ -162,7 +169,11 @@ gossip_dbus_set_presence (GossipDBus   *obj,
 		return FALSE;
 	}
 
-	gossip_app_set_presence (show, status);
+	if (G_STR_EMPTY (status)) {
+		gossip_app_set_presence (show, NULL);
+	} else {
+		gossip_app_set_presence (show, status);
+	}
 
 	return TRUE;
 }
