@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,13 +16,17 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_NEW_MESSAGE_DIALOG_H__
 #define __GOSSIP_NEW_MESSAGE_DIALOG_H__
 
-#include <gtk/gtkwindow.h>
+G_BEGIN_DECLS
 
 void gossip_new_message_dialog_show (GtkWindow *parent);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_NEW_MESSAGE_DIALOG_H__ */
